@@ -735,24 +735,43 @@ def make_tphase(freq,  min_en=3., max_en=70., t_step=1000, n_bins=32, orbitfile=
             run()
             shutil.copy('%s%s_TPHASE.fits' % (tt, unit), '%s%s_TPHASE_%03d.fits' % (tt, unit, n_bins))
 
-def rebin_matrix(e_min, e_max, pp_input, dpp_input, min_s_n = 50, only_pulsed=False, use_counts=False, background=None):
+
+
+
+
+
+
+
+def rebin_matrix(e_min_matrix, e_max_matrix, pp_input_matrix, dpp_input_matrix, min_s_n = 50, only_pulsed=False, use_counts=False, background_matrix=None, flip = False):
     '''
 
-    :param e_min: array with minimum energy of each bin
-    :param e_max: array with maximum energy of each bin
-    :param pp: 2-d array with pulse profiles
-    :param dpp: 2-d array with pulse profile uncertainties
+    :param e_min_matrix: array with minimum energy of each bin
+    :param e_max_matrix: array with maximum energy of each bin
+    :param pp_input_matrix: 2-d array with pulse profiles
+    :param dpp_input_matrix: 2-d array with pulse profile uncertainties
     :param min_s_n: minimum S/N for rebin
     :param only_pulsed:  it subtracts the average value before rebinning
-    ;param background a couple (background, background_uncertainty)
+    ;param background_matrix a couple (background, background_uncertainty)
+    :param flip : boolean ; if True rebin matrix from the last channel
     :return:
     new_e_mins rebinned array with minimum energy of each bin
     new_e_maxs rebinned array with maximum energy of each bin
     new_pulses rebinned 2-d array with pulse profiles
     dnew_pulses rebinned 2-d array with pulse profile uncertainties
     (new_background rebinned 2-d array with background pulse profiles
-    dnew_background rebinned 2-d array with background pulse profiles uncertainties) - returned only is background is not None 
+    dnew_background rebinned 2-d array with background pulse profiles uncertainties) - returned only if background is not None !
     '''
+    e_min = copy.deepcopy(e_min_matrix)
+    e_max = copy.deepcopy(e_max_matrix)
+    pp_input = copy.deepcopy(pp_input_matrix)
+    dpp_input = copy.deepcopy(dpp_input_matrix)
+
+    if flip:
+        logger.info('Rebinning starts from highest energy/time channel')
+        e_min = np.flip(e_min)
+        e_max = np.flip(e_max)
+        pp_input = np.flip(pp_input)
+        dpp_input = np.flip(dpp_input)
 
     new_pulses = []
     dnew_pulses = []
@@ -764,9 +783,18 @@ def rebin_matrix(e_min, e_max, pp_input, dpp_input, min_s_n = 50, only_pulsed=Fa
     pp = copy.deepcopy(pp_input)
     dpp = copy.deepcopy(dpp_input)
 
-    if background is not None:
-        pp_b = copy.deepcopy(background[0])
-        dpp_b = copy.deepcopy(background[1])
+    if background_matrix is not None:
+        background = [[],[]]
+        background[0] = copy.deepcopy(background_matrix[0])
+        background[1] = copy.deepcopy(background_matrix[1])
+        if flip:
+            backk = np.flip(background[0])
+            dbackk = np.flip(background[1])
+            pp_b = copy.deepcopy(backk)
+            dpp_b = copy.deepcopy(dbackk)
+        else:
+            pp_b = copy.deepcopy(background[0])
+            dpp_b = copy.deepcopy(background[1])
 
         new_pulses_b = []
         dnew_pulses_b = []
@@ -774,7 +802,7 @@ def rebin_matrix(e_min, e_max, pp_input, dpp_input, min_s_n = 50, only_pulsed=Fa
     while i1 < len(e_min) - 1:
         p1 = copy.copy(pp[i1, :])
         dp1 = copy.copy(dpp[i1, :])
-        if background is not None:
+        if background_matrix is not None:
             p1_b = copy.copy(pp_b[i1, :])
             dp1_b = copy.copy(dpp_b[i1, :])
         logger.debug('%d' % i1)
@@ -795,15 +823,19 @@ def rebin_matrix(e_min, e_max, pp_input, dpp_input, min_s_n = 50, only_pulsed=Fa
                 else:
                     new_pulses.append(p1/float(i2-i1))
                     dnew_pulses.append(dp1/float(i2-i1))
-                if background is not None:
+                if background_matrix is not None:
                     if use_counts:
                         new_pulses_b.append(p1_b)
                         dnew_pulses_b.append(dp1_b)
                     else:
                         new_pulses_b.append(p1_b/float(i2-i1))
                         dnew_pulses_b.append(dp1_b/float(i2-i1))
-                new_e_mins.append(e_min[i1])
-                new_e_maxs.append(e_max[i2 - 1])
+                if flip:
+                    new_e_mins.append(e_min[i2 - 1])
+                    new_e_maxs.append(e_max[i1])
+                else:
+                    new_e_mins.append(e_min[i1])
+                    new_e_maxs.append(e_max[i2 - 1])
                 logger.debug("Boom %f %d %d " % (s_n, i1, i2))
                 # print("Boom", s_n, i1, i2)
                 # print(np.mean(p1)/float(i2-i1), np.mean(dp1)/float(i2-i1))
@@ -817,7 +849,7 @@ def rebin_matrix(e_min, e_max, pp_input, dpp_input, min_s_n = 50, only_pulsed=Fa
 
                 p1 += pp[i2, :]
                 dp1 = np.sqrt(dp1 ** 2 + dpp[i2, :] ** 2)
-                if background is not None:
+                if background_matrix is not None:
                     p1_b += pp_b[i2, :]
                     dp1_b = np.sqrt(dp1_b ** 2 + dpp_b[i2, :] ** 2)
                     
@@ -832,10 +864,18 @@ def rebin_matrix(e_min, e_max, pp_input, dpp_input, min_s_n = 50, only_pulsed=Fa
 
     logger.info('We rebinned from %d to %d bins at a minimum S/N of %.1f' % (len(e_min), len(new_e_mins), min_s_n))
     # not sure this is a good practice
-    if background is None:
-        return np.array(new_e_mins), np.array(new_e_maxs), np.array(new_pulses), np.array(dnew_pulses)
+    if background_matrix is None:
+        if flip:
+            return np.flip(np.array(new_e_mins)), np.flip(np.array(new_e_maxs)), np.flip(np.array(new_pulses)), np.flip(
+                np.array(dnew_pulses))
+        else:
+            return np.array(new_e_mins), np.array(new_e_maxs), np.array(new_pulses), np.array(dnew_pulses)
     else:
-        return np.array(new_e_mins), np.array(new_e_maxs), np.array(new_pulses), np.array(dnew_pulses), np.array(new_pulses_b), np.array(dnew_pulses_b)
+        if flip:
+            return np.flip(np.array(new_e_mins)), np.flip(np.array(new_e_maxs)), np.flip(np.array(new_pulses)), np.flip(np.array(dnew_pulses)), np.flip(np.array(new_pulses_b)), np.flip(np.array(dnew_pulses_b))
+        else:
+            return np.array(new_e_mins), np.array(new_e_maxs), np.array(new_pulses), np.array(dnew_pulses), np.array(new_pulses_b), np.array(dnew_pulses_b)
+
 
 def pulsed_fraction_area(c, dc, background=None, background_error=None): 
     """It computes the pulsed fraction with the Area method (ref. )
@@ -976,7 +1016,7 @@ def subtract_background(a0, background, background_error):
         a0_out = a0 - background_level
         if a0 == 0:
             a0_out = background_level
-            logger.warning('continuum level is zero')
+            logger.debug('continuum level is zero')
             #raise Exception
     return a0_out
 
